use serde::{Serialize, Deserialize, Deserializer};
use anyhow::{Result};
// use std::str::FromStr;
use crate::defs::{
  Cluster,
  LoginUser,
  SSHAccess,
  NetworkInterface,
  StorageDevice,
	Cntrllr,
};
use crate::clouds::defs::{TskSrvc,App};

#[derive(Eq, PartialEq, Copy, Clone, Debug, Serialize,  Deserialize)]
#[allow(non_camel_case_types)]
pub enum ProviderServerPlan {
  upcloud_1xCPU_1GB,
  upcloud_1xCPU_2GB,
  upcloud_2xCPU_4GB,
  upcloud_4xCPU_8GB,
  none,
}

impl Default for ProviderServerPlan {
 fn default() -> Self {
    ProviderServerPlan::upcloud_2xCPU_4GB
 }
}
impl ProviderServerPlan {
 pub fn get_plan (&self) -> String {
   match self {
    ProviderServerPlan::upcloud_1xCPU_1GB => String::from("1xCPU-1GB"),
    ProviderServerPlan::upcloud_1xCPU_2GB => String::from("1xCPU-2GB"),
    ProviderServerPlan::upcloud_2xCPU_4GB => String::from("2xCPU-4GB"),
    ProviderServerPlan::upcloud_4xCPU_8GB => String::from("4xCPU-8GB"),
    ProviderServerPlan::none => String::from(""),
   }
 }
 pub fn set_plan (plan: String) -> ProviderServerPlan {
   match plan.as_str() {
    "1xCPU-1GB" => ProviderServerPlan::upcloud_1xCPU_1GB,
    "upcloud_1xCPU_1GB" => ProviderServerPlan::upcloud_1xCPU_1GB,
    "1xCPU-2GB" => ProviderServerPlan::upcloud_1xCPU_2GB,
    "upcloud_1xCPU_2GB" => ProviderServerPlan::upcloud_1xCPU_2GB,
    "2xCPU-4GB" => ProviderServerPlan::upcloud_2xCPU_4GB,
    "upcloud_2xCPU_4GB" => ProviderServerPlan::upcloud_2xCPU_4GB,
    "4xCPU-8GB" => ProviderServerPlan::upcloud_4xCPU_8GB,
    "upcloud_4xCPU_8GB" => ProviderServerPlan::upcloud_4xCPU_8GB,
    "" => ProviderServerPlan::none,
    &_ => ProviderServerPlan::default(),
   }
 }
}

#[allow(non_camel_case_types)]
#[derive(Eq, PartialEq, Copy, Clone,Debug, Serialize,  Deserialize)]
pub enum ProviderZone {
  upcloud_nl_ams1,
  upcloud_es_mad1,
  upcloud_de_fra1,
  none,
}

impl Default for ProviderZone {
  fn default() -> Self {
    ProviderZone::upcloud_nl_ams1
  }
} 
impl ProviderZone {
 pub fn get_zone (&self) -> String {
   match self {
     ProviderZone::upcloud_nl_ams1 => String::from("nl-ams1"),
     ProviderZone::upcloud_es_mad1 => String::from("es-mad1"),
     ProviderZone::upcloud_de_fra1=> String::from("de-fra1"),
     ProviderZone::none => String::from(""),
   }
 }
 pub fn set_zone (zone: String) -> ProviderZone {
   match zone.as_str() {
    "nl-ams1" => ProviderZone::upcloud_nl_ams1,
    "upcloud_nl_ams1" => ProviderZone::upcloud_nl_ams1,
    "es-mad1" => ProviderZone::upcloud_es_mad1,
    "upcloud_es_mad1" => ProviderZone::upcloud_es_mad1,
    "de-fra1" => ProviderZone::upcloud_de_fra1,
    "upcloud_de_fra1" => ProviderZone::upcloud_de_fra1,
    "" => ProviderZone::none,
    &_ => ProviderZone::default(),
   }
 }
}

#[allow(non_snake_case)]
#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct ServerInstance {
  #[serde(default)]
  pub cluster: Cluster,
  pub hostname: String,
  pub title: String,
  #[serde(default)]
  pub loginUser: LoginUser,
  #[serde(default)]
  pub metadata: i8,
  #[serde(deserialize_with = "deserialize_plan")]
  pub plan: ProviderServerPlan,
  #[serde(deserialize_with = "deserialize_zone")]
  pub zone: ProviderZone,
  pub arch: String,
  #[serde(default)]
  pub sshAccess: SSHAccess,
  #[serde(default)]
  pub networks: Vec<NetworkInterface>,
  #[serde(default,rename(deserialize = "storages"))]
  pub storageDevices: Vec<StorageDevice>,
  #[serde(default)]
  pub timeZone: String,
  #[serde(default)]
  pub tags: Vec<String>,
  #[serde(default)]
  pub useFloatIP: bool,
  #[serde(default)]
  pub userData: Option<String>,
  pub tsksrvcs: Vec<TskSrvc>,
  pub apps: Vec<App>,
  pub clients: Vec<String>,
  pub rules: Vec<String>,
  pub backup: Vec<String>,
}

fn deserialize_plan<'de, D>(deserializer: D) -> Result<ProviderServerPlan, D::Error>
where D: Deserializer<'de> {
    let buf = String::deserialize(deserializer)?;
    // let res = String::from_str(&buf).map_err(serde::de::Error::custom) {
    // .unwrap_or_else(|| serde::de::Error::custom);
    Ok(ProviderServerPlan::set_plan(buf))
}

fn deserialize_zone<'de, D>(deserializer: D) -> Result<ProviderZone, D::Error>
where D: Deserializer<'de> {
    let buf = String::deserialize(deserializer)?;
    Ok(ProviderZone::set_zone(buf))
}

impl Default for ServerInstance {
  fn default() -> Self {
    let net: NetworkInterface = NetworkInterface::default();
    let net_basic: NetworkInterface = NetworkInterface::upcloud_basic();
    let storage = StorageDevice::upcloud_storage_device_ubuntu20(25);
    Self {
        cluster: Cluster::default(),
        hostname: String::from(""),
        title: String::from(""),
        loginUser: LoginUser::default(),
        metadata: 1,
        plan: ProviderServerPlan::upcloud_1xCPU_1GB,
        zone: ProviderZone::upcloud_nl_ams1,
        arch: String::from("linux_amd64"),
        sshAccess: SSHAccess::default(),
        networks: vec![ net, net_basic ],
        storageDevices: vec![ storage ],
        timeZone: String::from("UTC"),
        tags: Vec::new(),
        useFloatIP: false,
        userData: Some(String::from("")),
        tsksrvcs: Vec::new(),
        apps: Vec::new(),
        clients: Vec::new(),
        rules: Vec::new(),
        backup: Vec::new(),
    }  
  }
}

#[allow(non_snake_case)]
#[derive(Clone, Debug, Serialize,  Deserialize)]
pub struct SrvrCfg { 
  #[serde(default)]
  pub role: String,
  pub sshKeys: Vec<String>,
  #[serde(default)]
  pub sshKeyPath: String,
  pub storageDeviceSize: u16,
  #[serde(default)]
  pub tags: Vec<String>,
  #[serde(default)]
  pub useFloatIP: bool
}
impl Default for SrvrCfg {
  fn default() -> Self {
    Self {
      role: String::from(""),
      sshKeys: Vec::new(),
      sshKeyPath: String::from("/root/.ssh/id_rsa"),
      storageDeviceSize: 25,
      tags: Vec::new(),
      useFloatIP: false
    }
  }
}

#[allow(non_snake_case)]
#[derive(Clone, Debug, Serialize,  Deserialize, Default)]
pub struct ResourcesConfig {
  pub floatIP: Option<String>,
  #[serde(default)]
  pub hostPrefix: String,
  #[serde(default)]
  pub mainName: String,
  pub group: String,
  pub group_path: String,
  pub provider: Option<String>,
  pub domainName: Option<String>,
	pub cntrllrs: Vec<Cntrllr>,
  #[serde(default)]
  pub srvrcfg: SrvrCfg,
  #[serde(default)]
  pub servers: Vec<ServerInstance>,
}

#[allow(non_snake_case)]
#[derive(Clone, Debug, Serialize,  Deserialize, Default)]
pub struct ConfigResources {
  pub floatIP: Option<String>,
  #[serde(default)]
  pub hostPrefix: String,
  #[serde(default)]
  pub mainName: String,
  pub group: String,
  pub group_path: String,
  pub provider: Option<String>,
  pub domainName: Option<String>,
	pub cntrllrs: Vec<Cntrllr>,
  #[serde(default)]
  pub servers: Vec<ServerInstance>,
}

// Implementations & settings  


impl StorageDevice {
  pub fn upcloud_storage_device_ubuntu20 (size: u16) -> Self {
    Self {
      action: String::from("clone"),
      storage: String::from("01000000-0000-4000-8000-000030200200"),
      size: size,
			finalSize: size,
      partSizes: Some(String::from("")),
      makefs: Some(String::from("")),
      title: String::from("Ubuntu Server 20.04 LTS (Focal Fossa)"),
      tier: String::from("maxiopsa"),
      source: String::from(""),
    }
  }
}

impl NetworkInterface {
  pub fn upcloud_basic () -> Self {
    let mut basic =  NetworkInterface::default();
    basic.access = String::from("utility");
    basic
  }
  pub fn upcloud_sdn () -> Self {
    Self {
      access: String::from("private"),
      family: String::from("IPv4"),
      network: Some(String::from("")),
      ipaddress: Some(String::from("")),
    }
  }
}

impl ServerInstance {
  pub fn server (plan: ProviderServerPlan, zone: ProviderZone, size: u16, arch: &str) -> Self {
    let net: NetworkInterface = NetworkInterface::default();
    let net_basic: NetworkInterface = NetworkInterface::upcloud_basic();
    let storage = StorageDevice::upcloud_storage_device_ubuntu20(size);
    Self {
      cluster: Cluster::default(),
      hostname: String::from(""),
      title: String::from(""),
      loginUser: LoginUser::default(),
      metadata: 1,
      plan,
      zone,
      arch: String::from(arch),
      sshAccess: SSHAccess::default(),
      networks: vec![ net, net_basic ],
      storageDevices: vec![ storage ],
      timeZone: String::from("UTC"),
      tags: Vec::new(),
      useFloatIP: false,
      userData: Some(String::from("")),
      tsksrvcs: Vec::new(),
      apps: Vec::new(),
      clients: Vec::new(),
      rules: Vec::new(),
      backup: Vec::new(),
    }  
  }
}
